/*
 * Copyright (C) 2005 Tilman Sauerbeck (tilman at code-monkey de)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ruby.h>
#include <stdbool.h>

#include <Evas.h>

#include "rb_evas_main.h"
#include "rb_evas.h"
#include "rb_evas_object.h"

#define CALLBACK_HANDLER_FUNC(name) \
	static void on_##name (void *data, Evas *evas, \
	                       Evas_Object *evas_obj, void *event) \
{ \
	VALUE tmp = (VALUE) event, klass, cb = (VALUE) data, ev; \
\
	klass = rb_hash_aref (event_classes, INT2FIX (EVAS_CALLBACK_##name)); \
	if (!NIL_P (klass)) { \
		ev = rb_class_new_instance (1, &tmp, klass); \
		rb_funcall (cb, rb_intern ("call"), 1, ev); \
	} else \
		rb_funcall (cb, rb_intern ("call"), 0); \
}

#define CALLBACK_HANDLER_METHOD(callback) \
	VALUE cb; \
	GET_OBJ (self, RbEvasObject, e); \
\
	if (!rb_block_given_p ()) \
		return Qnil; \
\
	cb = rb_block_proc (); \
\
	if (NIL_P (e->callbacks)) \
		e->callbacks = rb_ary_new (); \
\
	rb_ary_push (e->callbacks, cb); \
\
	evas_object_event_callback_add (e->real, EVAS_CALLBACK_##callback, \
	                                on_##callback, (void *) cb); \
\
	return Qnil;

#define CALLBACK_REGISTER(name, callback, clsname) \
	rb_define_method (cEvasObject, "on_"#name, c_on_##name, 0); \
\
	c = rb_define_class_under (mEvas, (clsname), cEvent); \
	rb_define_private_method (rb_singleton_class (c), "new", NULL, 0); \
	rb_define_private_method (c, "initialize", c_ev_##name##_init, 1); \
\
	rb_hash_aset (event_classes, INT2FIX (EVAS_CALLBACK_##callback), c);

#define CALLBACK_REGISTER2(name) \
	rb_define_method (cEvasObject, "on_"#name, c_on_##name, 0);

#define TO_STR(s) \
	(s) ? rb_str_new2 ((s)) : Qnil

static VALUE event_classes, cPos;

static VALUE c_ev_mouse_in_init (VALUE self, VALUE ev)
{
	VALUE argv[4];
	Evas_Event_Mouse_In *e = (Evas_Event_Mouse_In *) ev;

	argv[0] = INT2FIX ((int) e->output.x);
	argv[1] = INT2FIX ((int) e->output.y);
	argv[2] = INT2FIX ((int) e->canvas.x);
	argv[3] = INT2FIX ((int) e->canvas.y);

	rb_iv_set (self, "@position",
	           rb_class_new_instance (4, argv, cPos));
	rb_iv_set (self, "@buttons", INT2FIX (e->buttons));

	return self;
}

static VALUE c_ev_mouse_out_init (VALUE self, VALUE ev)
{
	VALUE argv[4];
	Evas_Event_Mouse_Out *e = (Evas_Event_Mouse_Out *) ev;

	argv[0] = INT2FIX ((int) e->output.x);
	argv[1] = INT2FIX ((int) e->output.y);
	argv[2] = INT2FIX ((int) e->canvas.x);
	argv[3] = INT2FIX ((int) e->canvas.y);

	rb_iv_set (self, "@position",
	           rb_class_new_instance (4, argv, cPos));
	rb_iv_set (self, "@buttons", INT2FIX (e->buttons));

	return self;
}

static VALUE c_ev_mouse_down_init (VALUE self, VALUE ev)
{
	VALUE argv[4];
	Evas_Event_Mouse_Down *e = (Evas_Event_Mouse_Down *) ev;

	argv[0] = INT2FIX ((int) e->output.x);
	argv[1] = INT2FIX ((int) e->output.y);
	argv[2] = INT2FIX ((int) e->canvas.x);
	argv[3] = INT2FIX ((int) e->canvas.y);

	rb_iv_set (self, "@position",
	           rb_class_new_instance (4, argv, cPos));
	rb_iv_set (self, "@button", INT2FIX (e->button));

	return self;
}

static VALUE c_ev_mouse_up_init (VALUE self, VALUE ev)
{
	VALUE argv[4];
	Evas_Event_Mouse_Up *e = (Evas_Event_Mouse_Up *) ev;

	argv[0] = INT2FIX ((int) e->output.x);
	argv[1] = INT2FIX ((int) e->output.y);
	argv[2] = INT2FIX ((int) e->canvas.x);
	argv[3] = INT2FIX ((int) e->canvas.y);

	rb_iv_set (self, "@position",
	           rb_class_new_instance (4, argv, cPos));
	rb_iv_set (self, "@button", INT2FIX (e->button));

	return self;
}

static VALUE c_ev_mouse_move_init (VALUE self, VALUE ev)
{
	VALUE argv[4];
	Evas_Event_Mouse_Move *e = (Evas_Event_Mouse_Move *) ev;

	argv[0] = INT2FIX ((int) e->cur.output.x);
	argv[1] = INT2FIX ((int) e->cur.output.y);
	argv[2] = INT2FIX ((int) e->cur.canvas.x);
	argv[3] = INT2FIX ((int) e->cur.canvas.y);

	rb_iv_set (self, "@current",
	           rb_class_new_instance (4, argv, cPos));

	argv[0] = INT2FIX ((int) e->prev.output.x);
	argv[1] = INT2FIX ((int) e->prev.output.y);
	argv[2] = INT2FIX ((int) e->prev.canvas.x);
	argv[3] = INT2FIX ((int) e->prev.canvas.y);

	rb_iv_set (self, "@previous",
	           rb_class_new_instance (4, argv, cPos));
	rb_iv_set (self, "@buttons", INT2FIX (e->buttons));

	return self;
}

static VALUE c_ev_mouse_wheel_init (VALUE self, VALUE ev)
{
	VALUE argv[4];
	Evas_Event_Mouse_Wheel *e = (Evas_Event_Mouse_Wheel *) ev;

	argv[0] = INT2FIX ((int) e->output.x);
	argv[1] = INT2FIX ((int) e->output.y);
	argv[2] = INT2FIX ((int) e->canvas.x);
	argv[3] = INT2FIX ((int) e->canvas.y);

	rb_iv_set (self, "@position",
	           rb_class_new_instance (4, argv, cPos));
	rb_iv_set (self, "@direction", INT2FIX (e->direction));
	rb_iv_set (self, "@z", INT2FIX (e->z));

	return self;
}

static VALUE c_ev_key_down_init (VALUE self, VALUE ev)
{
	Evas_Event_Key_Down *e = (Evas_Event_Key_Down *) ev;

	rb_iv_set (self, "@keyname", TO_STR (e->keyname));
	rb_iv_set (self, "@key", TO_STR (e->key));
	rb_iv_set (self, "@string", TO_STR (e->string));
	rb_iv_set (self, "@compose", TO_STR (e->compose));

	return self;
}

static VALUE c_ev_key_up_init (VALUE self, VALUE ev)
{
	Evas_Event_Key_Up *e = (Evas_Event_Key_Up *) ev;

	rb_iv_set (self, "@keyname", TO_STR (e->keyname));
	rb_iv_set (self, "@key", TO_STR (e->key));
	rb_iv_set (self, "@string", TO_STR (e->string));
	rb_iv_set (self, "@compose", TO_STR (e->compose));

	return self;
}

CALLBACK_HANDLER_FUNC (MOUSE_IN);
CALLBACK_HANDLER_FUNC (MOUSE_OUT);
CALLBACK_HANDLER_FUNC (MOUSE_DOWN);
CALLBACK_HANDLER_FUNC (MOUSE_UP);
CALLBACK_HANDLER_FUNC (MOUSE_MOVE);
CALLBACK_HANDLER_FUNC (MOUSE_WHEEL);
CALLBACK_HANDLER_FUNC (KEY_DOWN);
CALLBACK_HANDLER_FUNC (KEY_UP);
CALLBACK_HANDLER_FUNC (FOCUS_IN);
CALLBACK_HANDLER_FUNC (FOCUS_OUT);
CALLBACK_HANDLER_FUNC (SHOW);
CALLBACK_HANDLER_FUNC (HIDE);
CALLBACK_HANDLER_FUNC (MOVE);
CALLBACK_HANDLER_FUNC (RESIZE);
CALLBACK_HANDLER_FUNC (RESTACK);

static VALUE c_on_mouse_in (VALUE self)
{
	CALLBACK_HANDLER_METHOD (MOUSE_IN);
}

static VALUE c_on_mouse_out (VALUE self)
{
	CALLBACK_HANDLER_METHOD (MOUSE_OUT);
}

static VALUE c_on_mouse_down (VALUE self)
{
	CALLBACK_HANDLER_METHOD (MOUSE_DOWN);
}

static VALUE c_on_mouse_up (VALUE self)
{
	CALLBACK_HANDLER_METHOD (MOUSE_UP);
}

static VALUE c_on_mouse_move (VALUE self)
{
	CALLBACK_HANDLER_METHOD (MOUSE_MOVE);
}

static VALUE c_on_mouse_wheel (VALUE self)
{
	CALLBACK_HANDLER_METHOD (MOUSE_WHEEL);
}

static VALUE c_on_key_down (VALUE self)
{
	CALLBACK_HANDLER_METHOD (KEY_DOWN);
}

static VALUE c_on_key_up (VALUE self)
{
	CALLBACK_HANDLER_METHOD (KEY_UP);
}

static VALUE c_on_focus_in (VALUE self)
{
	CALLBACK_HANDLER_METHOD (FOCUS_IN);
}

static VALUE c_on_focus_out (VALUE self)
{
	CALLBACK_HANDLER_METHOD (FOCUS_OUT);
}

static VALUE c_on_show (VALUE self)
{
	CALLBACK_HANDLER_METHOD (SHOW);
}

static VALUE c_on_hide (VALUE self)
{
	CALLBACK_HANDLER_METHOD (HIDE);
}

static VALUE c_on_move (VALUE self)
{
	CALLBACK_HANDLER_METHOD (MOVE);
}

static VALUE c_on_resize (VALUE self)
{
	CALLBACK_HANDLER_METHOD (RESIZE);
}

static VALUE c_on_restack (VALUE self)
{
	CALLBACK_HANDLER_METHOD (RESTACK);
}

static VALUE c_ev_init (VALUE argc, VALUE argv, VALUE self)
{
	return self;
}

static VALUE c_pos_init (VALUE self, VALUE output_x, VALUE output_y,
                         VALUE canvas_x, VALUE canvas_y)
{
	rb_iv_set (self, "@output_x", output_x);
	rb_iv_set (self, "@output_y", output_y);
	rb_iv_set (self, "@canvas_x", canvas_x);
	rb_iv_set (self, "@canvas_y", canvas_y);

	return self;
}

void Init_EvasObjectEvents (void)
{
	VALUE cEvent, c;

	event_classes = rb_hash_new ();
	rb_global_variable (&event_classes);

	cEvent = rb_define_class_under (mEvas, "EvasObjectEvent", rb_cObject);
	rb_define_private_method (rb_singleton_class (cEvent), "new", NULL, 0);
	rb_define_private_method (cEvent, "initialize", c_ev_init, -1);

	cPos = rb_define_class_under (cEvent, "Position", rb_cObject);
	rb_define_private_method (rb_singleton_class (cPos), "new", NULL, 0);
	rb_define_private_method (cPos, "initialize", c_pos_init, 4);
	rb_define_attr (cPos, "output_x", 1, 0);
	rb_define_attr (cPos, "output_y", 1, 0);
	rb_define_attr (cPos, "canvas_x", 1, 0);
	rb_define_attr (cPos, "canvas_y", 1, 0);

	CALLBACK_REGISTER (mouse_in, MOUSE_IN, "MouseInEvent");
	rb_define_attr (c, "buttons", 1, 0);
	rb_define_attr (c, "position", 1, 0);

	CALLBACK_REGISTER (mouse_out, MOUSE_OUT, "MouseOutEvent");
	rb_define_attr (c, "buttons", 1, 0);
	rb_define_attr (c, "position", 1, 0);

	CALLBACK_REGISTER (mouse_down, MOUSE_DOWN, "MouseDownEvent");
	rb_define_attr (c, "button", 1, 0);
	rb_define_attr (c, "position", 1, 0);

	CALLBACK_REGISTER (mouse_up, MOUSE_UP, "MouseUpEvent");
	rb_define_attr (c, "button", 1, 0);
	rb_define_attr (c, "position", 1, 0);

	CALLBACK_REGISTER (mouse_move, MOUSE_MOVE, "MouseMoveEvent");
	rb_define_attr (c, "current", 1, 0);
	rb_define_attr (c, "previous", 1, 0);
	rb_define_attr (c, "buttons", 1, 0);

	CALLBACK_REGISTER (mouse_wheel, MOUSE_WHEEL, "MouseWheelEvent");
	rb_define_attr (c, "direction", 1, 0);
	rb_define_attr (c, "z", 1, 0);
	rb_define_attr (c, "position", 1, 0);

	CALLBACK_REGISTER (key_down, KEY_DOWN, "KeyDownEvent");
	rb_define_attr (c, "keyname", 1, 0);
	rb_define_attr (c, "key", 1, 0);
	rb_define_attr (c, "string", 1, 0);
	rb_define_attr (c, "compose", 1, 0);

	CALLBACK_REGISTER (key_up, KEY_UP, "KeyUpEvent");
	rb_define_attr (c, "keyname", 1, 0);
	rb_define_attr (c, "key", 1, 0);
	rb_define_attr (c, "string", 1, 0);
	rb_define_attr (c, "compose", 1, 0);

	CALLBACK_REGISTER2 (focus_in);
	CALLBACK_REGISTER2 (focus_out);
	CALLBACK_REGISTER2 (show);
	CALLBACK_REGISTER2 (hide);
	CALLBACK_REGISTER2 (move);
	CALLBACK_REGISTER2 (resize);
	CALLBACK_REGISTER2 (restack);
}
