/*
 * Copyright (C) 2004 Tilman Sauerbeck (tilman at code-monkey de)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ruby.h>

#include <Evas.h>

#include "rb_evas_main.h"
#include "rb_evas.h"
#include "rb_evas_object.h"

#define DEF_CONST(mod, prefix, name) \
	rb_define_const ((mod), #name, INT2FIX (prefix##name));

#define GET_COLOR_METHOD(name) \
	int r = 0, g = 0, b = 0, a = 0; \
\
	GET_OBJ (self, RbEvasObject, e); \
\
	evas_object_text_##name##_color_get (e->real, &r, &g, &b, &a); \
\
	return rb_ary_new3 (4, \
	                    INT2FIX (r), INT2FIX (g), \
	                    INT2FIX (b), INT2FIX (a));

#define SET_COLOR_METHOD(name) \
	GET_OBJ (self, RbEvasObject, e); \
\
	Check_Type (r, T_FIXNUM); \
	Check_Type (g, T_FIXNUM); \
	Check_Type (b, T_FIXNUM); \
	Check_Type (a, T_FIXNUM); \
\
	evas_object_text_##name##_color_set (e->real, \
	                                     FIX2INT (r), FIX2INT (g), \
	                                     FIX2INT (b), FIX2INT (a)); \
\
	return Qnil;

/*
 * call-seq:
 *  Evas::Text.new(evas) => text
 *
 * Creates an Evas::Text object.
 */
static VALUE c_init (VALUE self, VALUE evas)
{
	CHECK_CLASS (evas, cEvas);
	GET_OBJ (evas, RbEvas, e);
	GET_OBJ (self, RbEvasObject, text);

	text->real = evas_object_text_add (e->real);

	rb_call_super (1, &evas);

	return self;
}

/*
 * call-seq:
 *  text.font_source => string or nil
 *
 * Returns the font source of <i>text</i>.
 */
static VALUE c_font_source_get (VALUE self)
{
	const char *tmp;

	GET_OBJ (self, RbEvasObject, e);

	if (!(tmp = evas_object_text_font_source_get (e->real)))
		return Qnil;
	else
		return rb_str_new2 (tmp);
}

/*
 * call-seq:
 *  text.font_source(source)
 *
 * Sets the font source of <i>text</i>.
 */
static VALUE c_font_source_set (VALUE self, VALUE val)
{
	GET_OBJ (self, RbEvasObject, e);

	evas_object_text_font_source_set (e->real, StringValuePtr (val));

	return Qnil;
}

/*
 * call-seq:
 *  text.get_font => array
 *
 * Returns the font name and font size of <i>text</i>.
 *
 *  text.set_font("vera", 10) #=> nil
 *  text_get_font             #=> ["vera", 10]
 */
static VALUE c_get_font (VALUE self)
{
	const char *font = NULL;
	Evas_Font_Size size = 0;

	GET_OBJ (self, RbEvasObject, e);

	evas_object_text_font_get (e->real, &font, &size);

	return rb_ary_new3 (2, font ? rb_str_new2 (font) : Qnil,
	                    INT2FIX (size));
}

/*
 * call-seq:
 *  text.set_font(font, size) => nil
 *
 * Sets the font name and font size of <i>text</i>.
 *
 *  text.set_font("vera", 10) #=> nil
 */
static VALUE c_set_font (VALUE self, VALUE font, VALUE size)
{
	GET_OBJ (self, RbEvasObject, e);

	Check_Type (size, T_FIXNUM);

	evas_object_text_font_set (e->real, StringValuePtr (font),
	                           FIX2INT (size));

	return Qnil;
}

/*
 * call-seq:
 *  text.text => string
 *
 * Returns the text of <i>text</i>.
 */
static VALUE c_text_get (VALUE self)
{
	const char *tmp;

	GET_OBJ (self, RbEvasObject, e);

	if (!(tmp = evas_object_text_text_get (e->real)))
		return Qnil;
	else
		return rb_str_new2 (tmp);
}

/*
 * call-seq:
 *  text.text(string)
 *
 * Sets the text of <i>text</i>.
 */
static VALUE c_text_set (VALUE self, VALUE val)
{
	GET_OBJ (self, RbEvasObject, e);

	evas_object_text_text_set (e->real, StringValuePtr (val));

	return Qnil;
}

/*
 * call-seq:
 *  text.style => integer
 *
 * Returns the style of <i>text</i>.
 */
static VALUE c_style_get (VALUE self)
{
	GET_OBJ (self, RbEvasObject, e);

	return INT2FIX (evas_object_text_style_get (e->real));
}

/*
 * call-seq:
 *  text.style = integer
 *
 * Sets the style of <i>text</i>.
 */
static VALUE c_style_set (VALUE self, VALUE val)
{
	GET_OBJ (self, RbEvasObject, e);

	Check_Type (val, T_FIXNUM);

	evas_object_text_style_set (e->real, FIX2INT (val));

	return Qnil;
}

static VALUE c_style_pad_get (VALUE self)
{
	int l = 0, r = 0, t = 0, b = 0;

	GET_OBJ (self, RbEvasObject, e);

	evas_object_text_style_pad_get (e->real, &l, &r, &t, &b);

	return rb_ary_new3 (4,
	                    INT2FIX (l), INT2FIX (r),
	                    INT2FIX (r), INT2FIX (b));
}

/*
 * call-seq:
 *  e.get_shadow_color => array
 *
 * Returns the shadow color of <i>e</i>.
 *
 *  e.set_shadow_color(128, 128, 128, 0) #=> nil
 *  e.get_shadow_color                   #=> [128, 128, 128, 0]
 */
static VALUE c_get_shadow_color (VALUE self)
{
	GET_COLOR_METHOD (shadow);
}

/*
 * call-seq:
 *  e.set_shadow_color(r, g, b, a) => nil
 *
 * Sets the shadow color of <i>e</i>.
 *
 *  e.set_shadow_color(128, 128, 128, 0) #=> nil
 */
static VALUE c_set_shadow_color (VALUE self,
                                 VALUE r, VALUE g, VALUE b, VALUE a)
{
	SET_COLOR_METHOD (shadow);
}

/*
 * call-seq:
 *  e.get_glow_color => array
 *
 * Returns the glow color of <i>e</i>.
 *
 *  e.set_glow_color(128, 128, 128, 0) #=> nil
 *  e.get_glow_color                   #=> [128, 128, 128, 0]
 */
static VALUE c_get_glow_color (VALUE self)
{
	GET_COLOR_METHOD (glow);
}

/*
 * call-seq:
 *  e.set_glow_color(r, g, b, a) => nil
 *
 * Sets the glow color of <i>e</i>.
 *
 *  e.set_glow_color(128, 128, 128, 0) #=> nil
 */
static VALUE c_set_glow_color (VALUE self,
                                 VALUE r, VALUE g, VALUE b, VALUE a)
{
	SET_COLOR_METHOD (glow);
}

/*
 * call-seq:
 *  e.get_glow2_color => array
 *
 * Returns the glow2 color of <i>e</i>.
 *
 *  e.set_glow2_color(128, 128, 128, 0) #=> nil
 *  e.get_glow2_color                   #=> [128, 128, 128, 0]
 */
static VALUE c_get_glow2_color (VALUE self)
{
	GET_COLOR_METHOD (glow2);
}

/*
 * call-seq:
 *  e.set_glow2_color(r, g, b, a) => nil
 *
 * Sets the glow2 color of <i>e</i>.
 *
 *  e.set_glow2_color(128, 128, 128, 0) #=> nil
 */
static VALUE c_set_glow2_color (VALUE self,
                               VALUE r, VALUE g, VALUE b, VALUE a)
{
	SET_COLOR_METHOD (glow2);
}

/*
 * call-seq:
 *  e.get_outline_color => array
 *
 * Returns the outline color of <i>e</i>.
 *
 *  e.set_outline_color(128, 128, 128, 0) #=> nil
 *  e.get_outline_color                   #=> [128, 128, 128, 0]
 */
static VALUE c_get_outline_color (VALUE self)
{
	GET_COLOR_METHOD (outline);
}

/*
 * call-seq:
 *  e.set_outline_color(r, g, b, a) => nil
 *
 * Sets the outline color of <i>e</i>.
 *
 *  e.set_outline_color(128, 128, 128, 0) #=> nil
 */
static VALUE c_set_outline_color (VALUE self,
                                  VALUE r, VALUE g, VALUE b, VALUE a)
{
	SET_COLOR_METHOD (outline);
}

static VALUE c_ascent_get (VALUE self)
{
	GET_OBJ (self, RbEvasObject, e);

	return INT2FIX ((int) evas_object_text_ascent_get (e->real));
}

static VALUE c_descent_get (VALUE self)
{
	GET_OBJ (self, RbEvasObject, e);

	return INT2FIX ((int) evas_object_text_descent_get (e->real));
}

static VALUE c_max_ascent_get (VALUE self)
{
	GET_OBJ (self, RbEvasObject, e);

	return INT2FIX ((int) evas_object_text_max_ascent_get (e->real));
}

static VALUE c_max_descent_get (VALUE self)
{
	GET_OBJ (self, RbEvasObject, e);

	return INT2FIX ((int) evas_object_text_max_descent_get (e->real));
}

static VALUE c_advance_get (VALUE self)
{
	int h = 0, v = 0;

	GET_OBJ (self, RbEvasObject, e);

	h = (int) evas_object_text_horiz_advance_get (e->real);
	v = (int) evas_object_text_vert_advance_get (e->real);

	return rb_ary_new3 (2, INT2FIX (h), INT2FIX (v));
}

static VALUE c_inset_get (VALUE self)
{
	GET_OBJ (self, RbEvasObject, e);

	return INT2FIX ((int) evas_object_text_inset_get (e->real));
}

void Init_Text (void)
{
	VALUE c, c2;

	c = rb_define_class_under (mEvas, "Text", cEvasObject);

	rb_define_method (c, "initialize", c_init, 1);
	rb_define_method (c, "font_source", c_font_source_get, 0);
	rb_define_method (c, "font_source=", c_font_source_set, 1);
	rb_define_method (c, "get_font", c_get_font, 0);
	rb_define_method (c, "set_font", c_set_font, 2);
	rb_define_method (c, "text", c_text_get, 0);
	rb_define_method (c, "text=", c_text_set, 1);
	rb_define_method (c, "style", c_style_get, 0);
	rb_define_method (c, "style=", c_style_set, 1);
	rb_define_method (c, "style_pad", c_style_pad_get, 0);
	rb_define_method (c, "get_shadow_color", c_get_shadow_color, 0);
	rb_define_method (c, "set_shadow_color", c_set_shadow_color, 4);
	rb_define_method (c, "get_glow_color", c_get_glow_color, 0);
	rb_define_method (c, "set_glow_color", c_set_glow_color, 4);
	rb_define_method (c, "get_glow2_color", c_get_glow2_color, 0);
	rb_define_method (c, "set_glow2_color", c_set_glow2_color, 4);
	rb_define_method (c, "get_outline_color", c_get_outline_color, 0);
	rb_define_method (c, "set_outline_color", c_set_outline_color, 4);

	rb_define_method (c, "ascent", c_ascent_get, 0);
	rb_define_method (c, "descent", c_descent_get, 0);
	rb_define_method (c, "max_ascent", c_max_ascent_get, 0);
	rb_define_method (c, "max_descent", c_max_descent_get, 0);
	rb_define_method (c, "advance", c_advance_get, 0);
	rb_define_method (c, "inset", c_inset_get, 0);

	c2 = rb_define_class_under (mEvas, "Style", c);

	DEF_CONST (c2, EVAS_TEXT_STYLE_, PLAIN);
	DEF_CONST (c2, EVAS_TEXT_STYLE_, SHADOW);
	DEF_CONST (c2, EVAS_TEXT_STYLE_, OUTLINE);
	DEF_CONST (c2, EVAS_TEXT_STYLE_, SOFT_OUTLINE);
	DEF_CONST (c2, EVAS_TEXT_STYLE_, GLOW);
	DEF_CONST (c2, EVAS_TEXT_STYLE_, OUTLINE_SHADOW);
	DEF_CONST (c2, EVAS_TEXT_STYLE_, FAR_SHADOW);
	DEF_CONST (c2, EVAS_TEXT_STYLE_, OUTLINE_SOFT_SHADOW);
	DEF_CONST (c2, EVAS_TEXT_STYLE_, SOFT_SHADOW);
	DEF_CONST (c2, EVAS_TEXT_STYLE_, FAR_SOFT_SHADOW);
}
